#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /etc/cron.d/elma-elasticsearch.sh
#

RETVAL=0
#IFS="_"
OPTIONS=$@

SCRIPT_DESCRIPTION="elasticsearch index optimization"
SCRIPT_AUTHOR="Joerg Heinemann"

# ######### Load ELMA configuration parameter, global functions and script header #########
#
source /etc/elma/elma.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

#
# Removing old expired time-based indexes
# Removing any diag indexes
# Removing any 1970s-era log indexes, as these are a product of timestamp fail
#
# Do elasticsearch optimize on previous day index
# if $1 = all then optimize all indexes
#

LATESTARCHIVEDATE=`date --date ${ES_ARCHIVE_PERIOD}' days ago' '+%Y-%m-%d'`

echo  --- `date +%T.%N | cut -c 1-12` - show todays ${TODAY} elasticsearch indexes ---------------------------
echo
curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' |grep ${TODAY}
echo
echo  --- `date +%T.%N | cut -c 1-12` - show yesterdays ${YESTERDAY} elasticsearch indexes -----------------------
echo
curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' |grep ${YESTERDAY}
echo
echo  --- `date +%T.%N | cut -c 1-12` - show all elasticsearch indexes -----------------------------------------
echo
curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}'
echo

echo  --- `date +%T.%N | cut -c 1-12` - delete elasticsearch indexes -------------------------------------------
echo

# Before we do anything, let's get rid of any nasty 1970s-aera indexes we have floating around
#
curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' |grep 1970 | while read LINE
do
	echo  --- `date +%T.%N | cut -c 1-12` - delete index with screwed-up timestamps: $LINE
	echo
	curl $ES_AUTH -s -XDELETE $ES_URL/$LINE/
	echo
	echo
done

# Loop through all elasticsearch diag indexes and delete them
#
curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' |grep diag | while read LINE
do
	echo  --- `date +%T.%N | cut -c 1-12` - delete diag index: $LINE
	echo
	curl $ES_AUTH -s -XDELETE $ES_URL/$LINE/
	echo
	echo
done

# Loop through all elasticsearch marvel indexes and delete those with expired timestamps
#
curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' | grep .marvel | while read LINE
do
	INDEXDATE=$(echo $LINE | awk -F'-' '{ print $2 }' | awk -F'.' '{ print $1,"-",$2,"-",$3 }')
	(( $(date -d "${INDEXDATE}" +%s) < $(date -d "${LATESTARCHIVEDATE}" +%s) )) && RESULT="earlier" || RESULT="later"
	if [ $RESULT == "earlier" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - delete index with expired timestamps (< $LATESTARCHIVEDATE): $LINE"
		echo
		curl $ES_AUTH -s -XDELETE $ES_URL/$LINE/
		echo
		echo
	fi
done

# Loop through all elasticsearch rsyslog indexes and delete those with expired timestamps
#
curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' | grep rsyslog | while read LINE
do
	INDEXDATE=$(echo $LINE | awk -F'-' '{ print $2,"-",$3,"-",$4 }')
	(( $(date -d "${INDEXDATE}" +%s) < $(date -d "${LATESTARCHIVEDATE}" +%s) )) && RESULT="earlier" || RESULT="later"
	if [ $RESULT == "earlier" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - delete index with expired timestamps (< $LATESTARCHIVEDATE): $LINE"
		echo
		curl $ES_AUTH -s -XDELETE $ES_URL/$LINE/
		echo
		echo
	fi
done

echo  --- `date +%T.%N | cut -c 1-12` - optimize elasticsearch indexes -----------------------------------------
echo

if [ "$1" = "all" ]
then
	# Loop through all elasticsearch indexes except today and optimize them
	#
	curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' |grep -v ${TODAY} | grep -v nagios | while read LINE
	do
		echo  --- `date +%T.%N | cut -c 1-12` - optimizing: $LINE
		echo
		curl $ES_AUTH -XPOST $ES_URL/$LINE/_optimize?max_num_segments=2
		echo
		echo
	done
else
	# Loop through all yesterday elasticsearch indexes and optimize them
	#
	curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' |grep ${YESTERDAY} | grep -v nagios | while read LINE
	do
		echo  --- `date +%T.%N | cut -c 1-12` - optimizing: $LINE
		echo
		curl $ES_AUTH -XPOST $ES_URL/$LINE/_optimize?max_num_segments=2
		echo
		echo
	done
fi

echo  --- `date +%T.%N | cut -c 1-12` - exit -------------------------------------------------------------------
echo

# Remove unused data directory for default ElasticSearch cluster
# Don't know why this directory was created during the installation process
if [ -d $ES_DATA_DIR/elasticsearch ]
then
  echo "Remove unused data directory for default ElasticSearch cluster"
  echo "cron-daily: `date +%T.%N | cut -c 1-12` - Remove unused data directory for default ElasticSearch cluster" >> /var/log/elasticsearch/unused-data-directory.log
  rm -R $ES_DATA_DIR/elasticsearch
  echo
fi

echo  ===========================================================================================
echo
echo  "Last log archive:		$LATESTARCHIVEDATE"
echo  "Days in log archive:		$ES_ARCHIVE_PERIOD"
echo  "Actual number of indexes:	"`curl $ES_AUTH -s $ES_URL/_status?pretty=true |grep index |grep -v { |sort |uniq |awk -F\" '{print $4}' |wc -l`
echo
echo  ===========================================================================================
echo
